searchData={"items":[{"type":"module","title":"Repatch","doc":"Repatch is a library for efficient, ergonomic and concise mocking/patching in tests (or not tests)","ref":"Repatch.html"},{"type":"module","title":"Features - Repatch","doc":"1. Patch **any** function or macro (except NIF and BIF). You can even patch and call private functions in test, or Erlang modules\n\n2. Designed to work with `async: true`. Has 3 isolation levels for testing multi-processes scenarios.\n\n3. Does not require any explicit boilerplate or DI. Though, you are completely free to use it with Repatch!\n\n5. Powerful call history tracking.\n\n6. `super` and `real` to help you call real implementations of the module.\n\n7. Works with other test frameworks and even in non-testing environments like `iex` or remote shell.","ref":"Repatch.html#module-features"},{"type":"module","title":"Installation - Repatch","doc":"```elixir\ndef deps do\n  [\n    {:repatch, \"~> 1.0\"}\n  ]\nend\n```","ref":"Repatch.html#module-installation"},{"type":"module","title":"One-minute intro - Repatch","doc":"> for ExUnit users\n\n1. Add `Repatch.setup()` into your `test_helper.exs` file after the `ExUnit.start()`\n\n2. `use Repatch.ExUnit` in your test module\n\n3. Call `Repatch.patch` or `Repatch.fake` to change implementation of any function and any module.","ref":"Repatch.html#module-one-minute-intro"},{"type":"module","title":"For example - Repatch","doc":"```elixir\ndefmodule ThatsATest do\n  use ExUnit.Case, async: true\n  use Repatch.ExUnit\n\n  test \"that's not a MapSet.new\" do\n    Repatch.patch(MapSet, :new, fn ->\n      %{thats_not: :a_map_set}\n    end)\n\n    assert MapSet.new() == %{thats_not: :a_map_set}\n\n    assert Repatch.called?(MapSet, :new, 1)\n  end\nend\n```","ref":"Repatch.html#module-for-example"},{"type":"function","title":"Repatch.allow/3","doc":"Enables the `allowed` process to use the patch from `owner` process.\nWorks only for patches in shared mode.\nSee `t:allow_option/0` for available options.","ref":"Repatch.html#allow/3"},{"type":"function","title":"Repatch.allowances/1","doc":"Lists all allowances of specified process (or `self()` by default).\nWorks only when shared mode is enabled.\n\nPlease note that deep allowances are returned as the final allowed process.","ref":"Repatch.html#allowances/1"},{"type":"function","title":"Repatch.called?/4","doc":"Checks if the function call is present in the history or not.\nWorks with exact match on arguments or just an arity.\nWorks only when history is enabled in setup.\nSee `t:called_check_option/0` for available options.","ref":"Repatch.html#called?/4"},{"type":"function","title":"Repatch.cleanup/1","doc":"Cleans up current test process (or any other process) Repatch-state.\nIt is recommended to be called during the test exit.\nCheck out `Repatch.ExUnit` module which set up this callback up.","ref":"Repatch.html#cleanup/1"},{"type":"function","title":"Repatch.fake/3","doc":"Replaces functions implementation of the `real_module` with functions\nof the `fake_module`.\n\nSee `t:fake_option/0` for available options.","ref":"Repatch.html#fake/3"},{"type":"function","title":"Repatch.info/4","doc":"For debugging purposes only. Returns list of tags which indicates patch state of the specified function.","ref":"Repatch.html#info/4"},{"type":"function","title":"Repatch.owner/1","doc":"Lists current owner of the allowed process (or `self()` by default).\nWorks only when shared mode is enabled.\n\nPlease note that deep allowances are returned as the final owner of the process.","ref":"Repatch.html#owner/1"},{"type":"function","title":"Repatch.patch/4","doc":"Substitutes implementation of the function with a new one.\nStarts tracking history on all calls in the module too.\nSee `t:patch_option/0` for available options.\n\nBe aware that it recompiles the module if it was not patched or spied on before, which may take some time.","ref":"Repatch.html#patch/4"},{"type":"macro","title":"Repatch.private/1","doc":"Just a compiler-friendly wrapper to call private functions on the module.\nWorks only on calls in `Module.function(arg0, arg1, arg2)` format.","ref":"Repatch.html#private/1"},{"type":"macro","title":"Repatch.real/1","doc":"Use this on a call which would result only to unpatched versions of functions to be called on the whole stack of calls.\nWorks only on calls in `Module.function(arg0, arg1, arg2)` format.","ref":"Repatch.html#real/1"},{"type":"function","title":"Repatch.repatched?/4","doc":"Checks if function is patched in any (or some specific) mode.","ref":"Repatch.html#repatched?/4"},{"type":"function","title":"Repatch.restore/4","doc":"Removes any patch or fake on the specified function.\nSee `t:restore_option/0` for available options.","ref":"Repatch.html#restore/4"},{"type":"function","title":"Repatch.restore_all/0","doc":"Clears all state of the `Repatch` including all patches, fakes and history, and reloads all\nold modules back, disabling history collection on them. It is not recommended to be called\nduring testing and it is suggested to be used only when Repatch is used in iex session.","ref":"Repatch.html#restore_all/0"},{"type":"function","title":"Repatch.setup/1","doc":"Setup function. Use it only once per test suite.\nSee `t:setup_option/0` for available options.\n\nIt is suggested to be put in the `test_helper.exs` after the `ExUnit.start()` line","ref":"Repatch.html#setup/1"},{"type":"function","title":"Repatch.spy/2","doc":"Cleans the existing history of current process calls to this module\nand starts tracking new history of all calls to the specified module.\n\nBe aware that it recompiles the module if it was not patched or spied on before, which may take some time.","ref":"Repatch.html#spy/2"},{"type":"macro","title":"Repatch.super/1","doc":"Use this on a call which would result only on one unpatched version of the function to be called.\nWorks only on calls in `Module.function(arg0, arg1, arg2)` format.","ref":"Repatch.html#super/1"},{"type":"type","title":"Repatch.allow_option/0","doc":"Options passed in the `allow/3` function.\n\n* `force` (boolean) — Whether to override existing allowance on the allowed process or not. Defaults to `false`.","ref":"Repatch.html#t:allow_option/0"},{"type":"type","title":"Repatch.called_check_option/0","doc":"Options passed in the `called?/4` function. When multiple options are specified, they are combined in logical AND fashion.\n\n* `by` (`:any` | pid) — what process called the function. Defaults to `self()`.\n* `at_least` (`:once` | integer) — at least how many times the function was called. Defaults to `:once`.\n* `exactly` (`:once` | integer) — exactly how many times the function was called.\n* `before` (`:erlang.monotonic_time/0` timestamp) — upper boundary of when the function was called.\n* `after` (`:erlang.monotonic_time/0` timestamp) — lower boundary of when the function was called.","ref":"Repatch.html#t:called_check_option/0"},{"type":"type","title":"Repatch.fake_option/0","doc":"Options passed in the `fake/3` function.\n\n* `ignore_forbidden_module` (boolean) — Whether to ignore the warning about forbidden module is being spied. Defaults to `false`.\n* `force` (boolean) — Whether to override existing patches and fakes. Defaults to `false`.\n* `mode` (`:local` | `:shared` | `:global`) — What mode to use for the fake. See `t:mode/0` for more info. Defaults to `:local`.","ref":"Repatch.html#t:fake_option/0"},{"type":"type","title":"Repatch.mode/0","doc":"Mode of the patch or fake","ref":"Repatch.html#t:mode/0"},{"type":"type","title":"Repatch.patch_option/0","doc":"Options passed in the `patch/4` function.\n\n* `ignore_forbidden_module` (boolean) — Whether to ignore the warning about forbidden module is being spied. Defaults to `false`.\n* `force` (boolean) — Whether to override existing patches and fakes. Defaults to `false`.\n* `mode` (`:local` | `:shared` | `:global`) — What mode to use for the patch. See `t:mode/0` for more info. Defaults to `:local`.","ref":"Repatch.html#t:patch_option/0"},{"type":"type","title":"Repatch.recompile_option/0","doc":"","ref":"Repatch.html#t:recompile_option/0"},{"type":"type","title":"Repatch.repatched_check_option/0","doc":"Options passed in the `repatched?/4` function.\n\n* `mode` (`:local` | `:shared` | `:global` | `:any`) — What mode to check the patch in. See `t:mode/0` for more info. Defaults to `:any`.","ref":"Repatch.html#t:repatched_check_option/0"},{"type":"type","title":"Repatch.restore_option/0","doc":"Options passed in the `restore/4` function.\n\n* `mode` (`:local` | `:shared` | `:global`) — What mode to remove the patch in. See `t:mode/0` for more info. Defaults to `:local`.","ref":"Repatch.html#t:restore_option/0"},{"type":"type","title":"Repatch.setup_option/0","doc":"Options passed in the `setup/1` function.\n\n* `enable_global` (boolean) — Whether to allow global mocks in test suites. Defaults to `false`.\n* `enable_shared` (boolean) — Whether to allow shared mocks in test suites. Defaults to `true`.\n* `enable_history` (boolean) — Whether to enable calls history tracking. Defaults to `true`.\n* `recompile` (list of modules) — What modules should be recompiled before test starts. Modules are recompiled lazily by default. Defaults to `[]`.\n* `ignore_forbidden_module` (boolean) — Whether to ignore the warning about forbidden module being recompiled. Works only when `recompile` is specified. Defaults to `false`.","ref":"Repatch.html#t:setup_option/0"},{"type":"type","title":"Repatch.spy_option/0","doc":"Options passed in the `spy/2` function.\n\n* `by` (pid) — What process history to clean. Defaults to `self()`.\n* `ignore_forbidden_module` (boolean) — Whether to ignore the warning about forbidden module is being spied. Defaults to `false`.","ref":"Repatch.html#t:spy_option/0"},{"type":"type","title":"Repatch.tag/0","doc":"","ref":"Repatch.html#t:tag/0"},{"type":"module","title":"Repatch.ExUnit","doc":"An `ExUnit`-friendly helper to be used in testing. Just add\n\n```elixir\nuse Repatch.ExUnit\n```\n\nSomewhere after the `use ExUnit.Case` line and this helper will setup\ncleaning of `Repatch` state","ref":"Repatch.ExUnit.html"},{"type":"extras","title":"Repatch","doc":"# Repatch\n\n<!-- MDOC -->\n\nRepatch is a library for efficient, ergonomic and concise mocking/patching in tests (or not tests)","ref":"readme.html"},{"type":"extras","title":"Features - Repatch","doc":"1. Patch **any** function or macro (except NIF and BIF). You can even patch and call private functions in test, or Erlang modules\n\n2. Designed to work with `async: true`. Has 3 isolation levels for testing multi-processes scenarios.\n\n3. Does not require any explicit boilerplate or DI. Though, you are completely free to use it with Repatch!\n\n5. Powerful call history tracking.\n\n6. `super` and `real` to help you call real implementations of the module.\n\n7. Works with other test frameworks and even in non-testing environments like `iex` or remote shell.","ref":"readme.html#features"},{"type":"extras","title":"Installation - Repatch","doc":"```elixir\ndef deps do\n  [\n    {:repatch, \"~> 1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"One-minute intro - Repatch","doc":"> for ExUnit users\n\n1. Add `Repatch.setup()` into your `test_helper.exs` file after the `ExUnit.start()`\n\n2. `use Repatch.ExUnit` in your test module\n\n3. Call `Repatch.patch` or `Repatch.fake` to change implementation of any function and any module.","ref":"readme.html#one-minute-intro"},{"type":"extras","title":"For example - Repatch","doc":"```elixir\ndefmodule ThatsATest do\n  use ExUnit.Case, async: true\n  use Repatch.ExUnit\n\n  test \"that's not a MapSet.new\" do\n    Repatch.patch(MapSet, :new, fn ->\n      %{thats_not: :a_map_set}\n    end)\n\n    assert MapSet.new() == %{thats_not: :a_map_set}\n\n    assert Repatch.called?(MapSet, :new, 1)\n  end\nend\n```\n\n<!-- MDOC -->","ref":"readme.html#for-example"},{"type":"extras","title":"Further reading - Repatch","doc":"Please check out the [docs](https://hexdocs.pm/repatch) for all available features.","ref":"readme.html#further-reading"},{"type":"extras","title":"Special thanks - Repatch","doc":"To [`ihumanable`](https://github.com/ihumanable) for [`Patch`](https://hexdocs.pm/patch) library which was an inspiration and a good example for `Repatch`.","ref":"readme.html#special-thanks"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"1.0.0 - Changelog","doc":"First stable version, ready for production use.","ref":"changelog.html#1-0-0"},{"type":"extras","title":"0.0.1 - Changelog","doc":"Initial version.\n\nNot ready for production yet.\n\nJust name squatting with basic functionality.\n\nPrepare for changes","ref":"changelog.html#0-0-1"},{"type":"extras","title":"How it works","doc":"# How it works\n\nIn order to change implementation of the function of already compiled module, `Repatch`\nperforms recompilation of the module which substitutes every function in the module\nwith the call to dispatcher function which falls back to original implementation when\nno override is found in dispatcher.\n\nSo, there are two parts in `Repatch`: recompiler and dispatcher","ref":"how_it_works.html"},{"type":"extras","title":"Recompiler - How it works","doc":"Recompiler relies on abstract_forms of the module (that's why it's important to compile\nmodules with `debug_info` erlang/elixir compiler option). It basically takes\nevery function in the module and generates a super (aka original or old) version\nof the function with the original implementation except that it will have a different name now and\na function with the original name but a different implementation, which will call dispatcher or\npass the arguments to the first function if there's nothing in dispatcher. Plus, recompiler\nalso generates a special public function for every private one, which just passes call to the original\nprivate one","ref":"how_it_works.html#recompiler"},{"type":"extras","title":"Dispatcher - How it works","doc":"For every function in patched module, dispatcher will be called first. It will first\ninsert a history entry (if history is enabled) and then it will lookup into process dictionary for\nthe local patches. Then (if shared patches are enabled) it will look into shared patches ets tables\nto check if there are any shared functions or allowances. And then (if global patches are enabled)\nit will look into global patches table. Finally, when no patches were found, it will return value\nwhich would indicate that original implementation must be called.","ref":"how_it_works.html#dispatcher"},{"type":"extras","title":"Comments - How it works","doc":"This page describes that in worst case, dispatcher will call process dictionary and lookup into 3 ets tables.\nIt is okay for function which are called not very frequently during a test suite, but it is not good when function is\ncalled a lot of times during the test case (like recursive list traversal functions), it will introduce\nvisible overhead. To reduce it, one should disable unused modes and history or take other actions from \"Performance tips\" guide","ref":"how_it_works.html#comments"},{"type":"extras","title":"Isolation modes","doc":"# Isolation modes\n\n`Repatch` has 3 different isolation modes, and it is important to know when and how to use each one of them","ref":"modes.html"},{"type":"extras","title":"Local mode - Isolation modes","doc":"This mode is always enabled and work by applying patches only to the **current process**.\nThat means that any other process will be unable to execute the patch that the other process set.\nHowever, local patches do not affect other processes and it is __completely safe__ to use this mode with\n`async: true` testing option. This mode is also the fastest one in terms of runtime overhead.","ref":"modes.html#local-mode"},{"type":"extras","title":"Shared mode - Isolation modes","doc":"This mode is enabled by default but it can be disabled with `enable_shared: false` setting in the `Patch.setup/1` call.\nIt works by storing the patch for the **current process and any allowed process**. This mechanic is similar to testing\nwith allowances in `Ecto` and `Mox`, though it is slitghtly different. To allow other process to use shared\npatches of the current process or any other process, one should use `Repatch.allow/2`\n\nConsider this example\n\n```elixir\npid = Server.start()\n\n# Everything works as expected without patch\nassert %MapSet{} = Server.call(pid, MapSet, :new, [])\nassert %MapSet{} = MapSet.new()\n\n# Now we apply patch and prove that only current process sees it\nRepatch.patch(MapSet, :new, [mode: :shared], fn -> :hello end)\nassert %MapSet{} = Server.call(pid, MapSet, :new, [])\nassert :hello = MapSet.new()\n\n# And now we allow the server to have access to this patch and we see\n# that it resolves to the same patch as the current process\nRepatch.allow(self(), pid)\nassert :hello = Server.call(pid, MapSet, :new, [])\nassert :hello = MapSet.new()\n```\n\nDue to it's nature, it is also safe to use this mode with `async: true` testing option. However, if two\ntests running at the same time, will try to allow the same third process to call shared patches,\none of the `allow` calls will fail, so it's up to developer to make sure that processes are not shared\nbetween tests.","ref":"modes.html#shared-mode"},{"type":"extras","title":"Global mode - Isolation modes","doc":"This mode is disabled by default and must be enabled with `enable_global: true` setting in `Patch.setup` call.\nIt works by storing the patch for the **current process and any other process**. This mechanic is similar to testing\nwith `Patch` library or global mode in `Mox`. Other processes will be able to call the patch without any explicit\nallowance or anything. Therefore, this mode can be used with `async: true` only when it is guaranteed that\nit is okay if other processes may call the patched function or it is guaranteed that other processes\nwill never call the patched function.","ref":"modes.html#global-mode"},{"type":"extras","title":"Performance tips","doc":"# Performance tips\n\n`Repatch` was built with performance in mind and it tries to keep the runtime overhead as thin as possible.\nHere is a list of all actions you may take in order to improve performance of your tests.\n\nWhile some of these tips are very simple to adopt, others may require rewrites of your tests. But in any way,\ntreat these as **tips, not rules**, because `Repatch` must work correctly and efficiently without them.","ref":"performance_tips.html"},{"type":"extras","title":"Use setup with recompile option - Performance tips","doc":"When `Repatch.patch/4` or `Repatch.fake/3` is called on the target module for the first time, this module will be\nrecompiled. Depending on the size of the module, it may take some time. Therefore it is recommended to recompile these modules\nbefore starting the test suite (during the `Repatch.setup/1` call). It may not reduce overall test suite time, but it will\ndefinitely help with having more consistent results when tracking slowest test cases.","ref":"performance_tips.html#use-setup-with-recompile-option"},{"type":"extras","title":"Disable unused modes - Performance tips","doc":"`Repatch` has optimizations to not lookup into the dispatch tables for modes which are disabled.\nThis advice also applies to async tests. If you use `global` test mode in async, feel free to\ndisable `shared` mode and then return `shared` and disable `global` during the `on_exit` callback.","ref":"performance_tips.html#disable-unused-modes"},{"type":"extras","title":"Prefer local to shared to global - Performance tips","doc":"In contrast to other tools, `Repatch` was designed with `async: true` in mind, that's why\nshared and global modes perform more expensive lookups into shared memory, while local\none is the most efficient and simple.","ref":"performance_tips.html#prefer-local-to-shared-to-global"},{"type":"extras","title":"Disable history - Performance tips","doc":"When module is recompiled, history is enabled for every function in this module. And by default\nhistory tracks **calls to any function in the module from all processes**. However, it is\npossible to track calls to the patched function manually.","ref":"performance_tips.html#disable-history"},{"type":"extras","title":"For example - Performance tips","doc":"```elixir\nhistory_agent = Agent.start_link(fn -> [] end)\n\nRepatch.patch(MapSet, :new, fn list ->\n  result = Repatch.super(MapSet.new(list))\n  Agent.update(history_agent, fn h -> [{MapSet, :new, [list], result} | h] end)\nend)\n\nMapSet.new([1])\nMapSet.new([1, 2])\n\nhistory = Agent.get(history_agent, &Function.identity/1)\n\nassert Enum.any?(history, &match?({MapSet, :new, [1 | _], %MapSet{}}, &1))\n```","ref":"performance_tips.html#for-example"},{"type":"extras","title":"Cleanup 3rd-party processes - Performance tips","doc":"If you're using processes which will outlive the test suite, and these processes call the patched modules, you should call `Repatch.cleanup/1` on them from time to time to clean history.","ref":"performance_tips.html#cleanup-3rd-party-processes"},{"type":"extras","title":"Try not to patch Elixir and Erlang kernel functions - Performance tips","doc":"That means if you have a function which returns result of `Enum.reduce` call, try not","ref":"performance_tips.html#try-not-to-patch-elixir-and-erlang-kernel-functions"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}